gnunet-svn
[Top][All Lists]
Advanced

[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]

[taler-exchange] branch master updated: -skeletons for reserve control e


From: gnunet
Subject: [taler-exchange] branch master updated: -skeletons for reserve control endpoints
Date: Sat, 01 Oct 2022 23:06:30 +0200

This is an automated email from the git hooks/post-receive script.

grothoff pushed a commit to branch master
in repository exchange.

The following commit(s) were added to refs/heads/master by this push:
     new 2f1fb32e -skeletons for reserve control endpoints
2f1fb32e is described below

commit 2f1fb32e1c51b05361ddf3321c316e3d8c65ebfe
Author: Christian Grothoff <christian@grothoff.org>
AuthorDate: Sat Oct 1 23:06:24 2022 +0200

    -skeletons for reserve control endpoints
---
 contrib/uncrustify.sh                              |   2 +-
 .../taler-exchange-httpd_reserves_attest.c         | 295 ++++++++++++++++++
 ...ry.h => taler-exchange-httpd_reserves_attest.h} |  20 +-
 src/exchange/taler-exchange-httpd_reserves_close.c | 343 +++++++++++++++++++++
 ...ory.h => taler-exchange-httpd_reserves_close.h} |  20 +-
 .../taler-exchange-httpd_reserves_get_attest.c     | 146 +++++++++
 .../taler-exchange-httpd_reserves_history.h        |   5 +-
 src/exchange/taler-exchange-httpd_reserves_open.c  | 281 +++++++++++++++++
 ...tory.h => taler-exchange-httpd_reserves_open.h} |  20 +-
 9 files changed, 1095 insertions(+), 37 deletions(-)

diff --git a/contrib/uncrustify.sh b/contrib/uncrustify.sh
index 09abaf54..e8e05d3e 100755
--- a/contrib/uncrustify.sh
+++ b/contrib/uncrustify.sh
@@ -9,6 +9,6 @@ if ! uncrustify --version >/dev/null; then
   exit 1
 fi
 
-find "$DIR/../src" \( -name "*.cpp" -o -name "*.c" -o -name "*.h" \) | grep -v 
mustach \
+find "$DIR/../src" \( -name "*.cpp" -o -name "*.c" -o -name "*.h" \) \
   -exec uncrustify -c "$DIR/uncrustify.cfg" --replace --no-backup {} + \
   || true
diff --git a/src/exchange/taler-exchange-httpd_reserves_attest.c 
b/src/exchange/taler-exchange-httpd_reserves_attest.c
new file mode 100644
index 00000000..94399f07
--- /dev/null
+++ b/src/exchange/taler-exchange-httpd_reserves_attest.c
@@ -0,0 +1,295 @@
+/*
+  This file is part of TALER
+  Copyright (C) 2014-2022 Taler Systems SA
+
+  TALER is free software; you can redistribute it and/or modify it under the
+  terms of the GNU Affero General Public License as published by the Free 
Software
+  Foundation; either version 3, or (at your option) any later version.
+
+  TALER is distributed in the hope that it will be useful, but WITHOUT ANY
+  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+  A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more 
details.
+
+  You should have received a copy of the GNU Affero General Public License 
along with
+  TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
+*/
+/**
+ * @file taler-exchange-httpd_reserves_close.c
+ * @brief Handle /reserves/$RESERVE_PUB/close requests
+ * @author Florian Dold
+ * @author Benedikt Mueller
+ * @author Christian Grothoff
+ */
+#include "platform.h"
+#include <gnunet/gnunet_util_lib.h>
+#include <jansson.h>
+#include "taler_mhd_lib.h"
+#include "taler_json_lib.h"
+#include "taler_dbevents.h"
+#include "taler-exchange-httpd_keys.h"
+#include "taler-exchange-httpd_reserves_attest.h"
+#include "taler-exchange-httpd_responses.h"
+
+
+/**
+ * How far do we allow a client's time to be off when
+ * checking the request timestamp?
+ */
+#define TIMESTAMP_TOLERANCE \
+  GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_MINUTES, 15)
+
+
+/**
+ * Closure for #reserve_attest_transaction.
+ */
+struct ReserveAttestContext
+{
+  /**
+   * Public key of the reserve the inquiry is about.
+   */
+  const struct TALER_ReservePublicKeyP *reserve_pub;
+
+  /**
+   * Timestamp of the request.
+   */
+  struct GNUNET_TIME_Timestamp timestamp;
+
+  /**
+   * Client signature approving the request.
+   */
+  struct TALER_ReserveSignatureP reserve_sig;
+
+  /**
+   * Attest of the reserve, set in the callback.
+   */
+  struct TALER_EXCHANGEDB_ReserveAttest *rh;
+
+  /**
+   * Global fees applying to the request.
+   */
+  const struct TEH_GlobalFee *gf;
+
+  /**
+   * Current reserve balance.
+   */
+  struct TALER_Amount balance;
+};
+
+
+/**
+ * Send reserve attest to client.
+ *
+ * @param connection connection to the client
+ * @param rhc reserve attest to return
+ * @return MHD result code
+ */
+static MHD_RESULT
+reply_reserve_attest_success (struct MHD_Connection *connection,
+                              const struct ReserveAttestContext *rhc)
+{
+  const struct TALER_EXCHANGEDB_ReserveAttest *rh = rhc->rh;
+  json_t *json_attest;
+
+  json_attest = TEH_RESPONSE_compile_reserve_attest (rh);
+  if (NULL == json_attest)
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       
TALER_EC_GENERIC_JSON_ALLOCATION_FAILURE,
+                                       NULL);
+  return TALER_MHD_REPLY_JSON_PACK (
+    connection,
+    MHD_HTTP_OK,
+    TALER_JSON_pack_amount ("balance",
+                            &rhc->balance),
+    GNUNET_JSON_pack_array_steal ("attest",
+                                  json_attest));
+}
+
+
+/**
+ * Function implementing /reserves/$RID/attest transaction.  Given the public
+ * key of a reserve, return the associated transaction attest.  Runs the
+ * transaction logic; IF it returns a non-error code, the transaction logic
+ * MUST NOT queue a MHD response.  IF it returns an hard error, the
+ * transaction logic MUST queue a MHD response and set @a mhd_ret.  IF it
+ * returns the soft error code, the function MAY be called again to retry and
+ * MUST not queue a MHD response.
+ *
+ * @param cls a `struct ReserveAttestContext *`
+ * @param connection MHD request which triggered the transaction
+ * @param[out] mhd_ret set to MHD response status for @a connection,
+ *             if transaction failed (!); unused
+ * @return transaction status
+ */
+static enum GNUNET_DB_QueryStatus
+reserve_attest_transaction (void *cls,
+                            struct MHD_Connection *connection,
+                            MHD_RESULT *mhd_ret)
+{
+  struct ReserveAttestContext *rsc = cls;
+  enum GNUNET_DB_QueryStatus qs;
+
+  if (! TALER_amount_is_zero (&rsc->gf->fees.attest))
+  {
+    bool balance_ok = false;
+    bool idempotent = true;
+
+    qs = TEH_plugin->insert_attest_request (TEH_plugin->cls,
+                                            rsc->reserve_pub,
+                                            &rsc->reserve_sig,
+                                            rsc->timestamp,
+                                            &rsc->gf->fees.attest,
+                                            &balance_ok,
+                                            &idempotent);
+    if (GNUNET_DB_STATUS_HARD_ERROR == qs)
+    {
+      GNUNET_break (0);
+      *mhd_ret
+        = TALER_MHD_reply_with_error (connection,
+                                      MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                      TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                      "get_reserve_attest");
+    }
+    if (qs <= 0)
+    {
+      GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR == qs);
+      return qs;
+    }
+    if (! balance_ok)
+    {
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_CONFLICT,
+                                         
TALER_EC_EXCHANGE_WITHDRAW_ATTEST_ERROR_INSUFFICIENT_FUNDS,
+                                         NULL);
+    }
+    if (idempotent)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                  "Idempotent /reserves/attest request observed. Is caching 
working?\n");
+    }
+  }
+  qs = TEH_plugin->get_reserve_attest (TEH_plugin->cls,
+                                       rsc->reserve_pub,
+                                       &rsc->balance,
+                                       &rsc->rh);
+  if (GNUNET_DB_STATUS_HARD_ERROR == qs)
+  {
+    GNUNET_break (0);
+    *mhd_ret
+      = TALER_MHD_reply_with_error (connection,
+                                    MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                    TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                    "get_reserve_attest");
+  }
+  return qs;
+}
+
+
+MHD_RESULT
+TEH_handler_reserves_attest (struct TEH_RequestContext *rc,
+                             const struct TALER_ReservePublicKeyP *reserve_pub,
+                             const json_t *root)
+{
+  struct ReserveAttestContext rsc;
+  MHD_RESULT mhd_ret;
+  struct GNUNET_JSON_Specification spec[] = {
+    GNUNET_JSON_spec_timestamp ("request_timestamp",
+                                &rsc.timestamp),
+    GNUNET_JSON_spec_fixed_auto ("reserve_sig",
+                                 &rsc.reserve_sig),
+    GNUNET_JSON_spec_end ()
+  };
+  struct GNUNET_TIME_Timestamp now;
+
+  rsc.reserve_pub = reserve_pub;
+  {
+    enum GNUNET_GenericReturnValue res;
+
+    res = TALER_MHD_parse_json_data (rc->connection,
+                                     root,
+                                     spec);
+    if (GNUNET_SYSERR == res)
+    {
+      GNUNET_break (0);
+      return MHD_NO; /* hard failure */
+    }
+    if (GNUNET_NO == res)
+    {
+      GNUNET_break_op (0);
+      return MHD_YES; /* failure */
+    }
+  }
+  now = GNUNET_TIME_timestamp_get ();
+  if (! GNUNET_TIME_absolute_approx_eq (now.abs_time,
+                                        rsc.timestamp.abs_time,
+                                        TIMESTAMP_TOLERANCE))
+  {
+    GNUNET_break_op (0);
+    return TALER_MHD_reply_with_error (rc->connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_EXCHANGE_GENERIC_CLOCK_SKEW,
+                                       NULL);
+  }
+  {
+    struct TEH_KeyStateHandle *keys;
+
+    keys = TEH_keys_get_state ();
+    if (NULL == keys)
+    {
+      GNUNET_break (0);
+      GNUNET_JSON_parse_free (spec);
+      return TALER_MHD_reply_with_error (rc->connection,
+                                         MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                         
TALER_EC_EXCHANGE_GENERIC_KEYS_MISSING,
+                                         NULL);
+    }
+    rsc.gf = TEH_keys_global_fee_by_time (keys,
+                                          rsc.timestamp);
+  }
+  if (NULL == rsc.gf)
+  {
+    GNUNET_break (0);
+    return TALER_MHD_reply_with_error (rc->connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       
TALER_EC_EXCHANGE_GENERIC_BAD_CONFIGURATION,
+                                       NULL);
+  }
+  if (GNUNET_OK !=
+      TALER_wallet_reserve_attest_verify (rsc.timestamp,
+                                          &rsc.gf->fees.attest,
+                                          reserve_pub,
+                                          &rsc.reserve_sig))
+  {
+    GNUNET_break_op (0);
+    return TALER_MHD_reply_with_error (rc->connection,
+                                       MHD_HTTP_FORBIDDEN,
+                                       
TALER_EC_EXCHANGE_RESERVES_ATTEST_BAD_SIGNATURE,
+                                       NULL);
+  }
+  rsc.rh = NULL;
+  if (GNUNET_OK !=
+      TEH_DB_run_transaction (rc->connection,
+                              "get reserve attest",
+                              TEH_MT_REQUEST_OTHER,
+                              &mhd_ret,
+                              &reserve_attest_transaction,
+                              &rsc))
+  {
+    return mhd_ret;
+  }
+  if (NULL == rsc.rh)
+  {
+    return TALER_MHD_reply_with_error (rc->connection,
+                                       MHD_HTTP_NOT_FOUND,
+                                       
TALER_EC_EXCHANGE_RESERVES_STATUS_UNKNOWN,
+                                       NULL);
+  }
+  mhd_ret = reply_reserve_attest_success (rc->connection,
+                                          &rsc);
+  TEH_plugin->free_reserve_attest (TEH_plugin->cls,
+                                   rsc.rh);
+  return mhd_ret;
+}
+
+
+/* end of taler-exchange-httpd_reserves_attest.c */
diff --git a/src/exchange/taler-exchange-httpd_reserves_close.c 
b/src/exchange/taler-exchange-httpd_reserves_close.c
new file mode 100644
index 00000000..28666924
--- /dev/null
+++ b/src/exchange/taler-exchange-httpd_reserves_close.c
@@ -0,0 +1,343 @@
+/*
+  This file is part of TALER
+  Copyright (C) 2014-2022 Taler Systems SA
+
+  TALER is free software; you can redistribute it and/or modify it under the
+  terms of the GNU Affero General Public License as published by the Free 
Software
+  Foundation; either version 3, or (at your option) any later version.
+
+  TALER is distributed in the hope that it will be useful, but WITHOUT ANY
+  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+  A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more 
details.
+
+  You should have received a copy of the GNU Affero General Public License 
along with
+  TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
+*/
+/**
+ * @file taler-exchange-httpd_reserves_close.c
+ * @brief Handle /reserves/$RESERVE_PUB/close requests
+ * @author Florian Dold
+ * @author Benedikt Mueller
+ * @author Christian Grothoff
+ */
+#include "platform.h"
+#include <gnunet/gnunet_util_lib.h>
+#include <jansson.h>
+#include "taler_mhd_lib.h"
+#include "taler_json_lib.h"
+#include "taler_dbevents.h"
+#include "taler-exchange-httpd_keys.h"
+#include "taler-exchange-httpd_reserves_close.h"
+#include "taler-exchange-httpd_responses.h"
+
+
+/**
+ * How far do we allow a client's time to be off when
+ * checking the request timestamp?
+ */
+#define TIMESTAMP_TOLERANCE \
+  GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_MINUTES, 15)
+
+
+/**
+ * Closure for #reserve_close_transaction.
+ */
+struct ReserveCloseContext
+{
+  /**
+   * Public key of the reserve the inquiry is about.
+   */
+  const struct TALER_ReservePublicKeyP *reserve_pub;
+
+  /**
+   * Timestamp of the request.
+   */
+  struct GNUNET_TIME_Timestamp timestamp;
+
+  /**
+   * Client signature approving the request.
+   */
+  struct TALER_ReserveSignatureP reserve_sig;
+
+  /**
+   * Global fees applying to the request.
+   */
+  const struct TEH_GlobalFee *gf;
+
+  /**
+   * Amount that will be wired (after closing fees).
+   */
+  struct TALER_Amount wire_amount;
+
+  /**
+   * Where to wire the funds, may be NULL.
+   */
+  const char *payto_uri;
+
+  /**
+   * Hash of the @e payto_uri.
+   */
+  struct TALER_PaytoHashP h_payto;
+
+};
+
+
+/**
+ * Send reserve close to client.
+ *
+ * @param connection connection to the client
+ * @param rhc reserve close to return
+ * @return MHD result code
+ */
+static MHD_RESULT
+reply_reserve_close_success (struct MHD_Connection *connection,
+                             const struct ReserveCloseContext *rhc)
+{
+  const struct TALER_EXCHANGEDB_ReserveClose *rh = rhc->rh;
+
+  return TALER_MHD_REPLY_JSON_PACK (
+    connection,
+    MHD_HTTP_OK,
+    TALER_JSON_pack_amount ("wire_amount",
+                            &rhc->wire_amount));
+}
+
+
+/**
+ * Function implementing /reserves/$RID/close transaction.  Given the public
+ * key of a reserve, return the associated transaction close.  Runs the
+ * transaction logic; IF it returns a non-error code, the transaction logic
+ * MUST NOT queue a MHD response.  IF it returns an hard error, the
+ * transaction logic MUST queue a MHD response and set @a mhd_ret.  IF it
+ * returns the soft error code, the function MAY be called again to retry and
+ * MUST not queue a MHD response.
+ *
+ * @param cls a `struct ReserveCloseContext *`
+ * @param connection MHD request which triggered the transaction
+ * @param[out] mhd_ret set to MHD response status for @a connection,
+ *             if transaction failed (!); unused
+ * @return transaction status
+ */
+static enum GNUNET_DB_QueryStatus
+reserve_close_transaction (void *cls,
+                           struct MHD_Connection *connection,
+                           MHD_RESULT *mhd_ret)
+{
+  struct ReserveCloseContext *rcc = cls;
+  enum GNUNET_DB_QueryStatus qs;
+  struct TALER_Amount balance;
+  char *payto_uri;
+
+  qs = TEH_plugin->get_reserve_balance (TEH_plugin->cls,
+                                        rcc->reserve_pub,
+                                        &balance,
+                                        &payto_uri);
+  switch (qs)
+  {
+  case GNUNET_DB_STATUS_HARD_ERROR:
+    GNUNET_break (0);
+    *mhd_ret
+      = TALER_MHD_reply_with_error (connection,
+                                    MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                    TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                    "get_reserve_balance");
+    return qs;
+  case GNUNET_DB_STATUS_SOFT_ERROR:
+    return qs;
+  case GNUNET_DB_STATUS_SUCCESS_NO_RESULTS:
+    *mhd_ret
+      = TALER_MHD_reply_with_error (rc->connection,
+                                    MHD_HTTP_NOT_FOUND,
+                                    TALER_EC_EXCHANGE_RESERVES_STATUS_UNKNOWN,
+                                    NULL);
+    return GNUNET_DB_STATUS_HARD_ERROR;
+  }
+
+  if ( (NULL == rcc->payto_uri) &&
+       (NULL == payto_uri) )
+  {
+    *mhd_ret
+      = TALER_MHD_reply_with_error (connection,
+                                    MHD_HTTP_CONFLICT,
+                                    TALER_EC_RESERVE_CLOSE_NO_TARGET_ACCOUNT,
+                                    NULL);
+    return GNUNET_DB_STATUS_HARD_ERROR;
+  }
+
+  if ( (NULL != rcc->payto_uri) &&
+       ( (NULL == payto_uri) ||
+         (0 != strcmp (payto_uri,
+                       rcc->payto_uri)) ) )
+  {
+    struct TALER_EXCHANGEDB_KycStatus kyc;
+    struct TALER_PaytoHashP kyc_payto;
+
+    /* FIXME: also fetch KYC status from reserve
+       in query above, and if payto_uri specified
+       and KYC not yet done (check KYC triggers!),
+       fail with 451 kyc required! */
+    *mhd_ret
+      = TEH_RESPONSE_reply_kyc_required (rcc->connection,
+                                         &kyc_payto,
+                                         &kyc);
+    return GNUNET_DB_STATUS_HARD_ERROR;
+  }
+
+  if (NULL == rcc->payto_uri)
+    rcc->payto_uri = payto_uri;
+
+  if (0 >
+      TALER_amount_subtract (&rcc->wire_amount,
+                             &balance,
+                             &rcc->gf->fees.close))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Client attempted to close reserve with insufficient 
balance.\n");
+    GNUNET_assert (GNUNET_OK ==
+                   TALER_amount_set_zero (TEH_currency,
+                                          &rcc->wire_amount));
+    *mhd_ret = reply_reserve_close_success (rc->connection,
+                                            &rcc);
+    return GNUNET_DB_STATUS_HARD_ERROR;
+  }
+
+  {
+    qs = TEH_plugin->insert_close_request (TEH_plugin->cls,
+                                           rcc->reserve_pub,
+                                           payto_uri,
+                                           &rcc->reserve_sig,
+                                           rcc->timestamp,
+                                           &rcc->gf->fees.close,
+                                           &rcc->wire_amount);
+    GNUNET_free (payto_uri);
+    if (GNUNET_DB_STATUS_HARD_ERROR == qs)
+    {
+      GNUNET_break (0);
+      *mhd_ret
+        = TALER_MHD_reply_with_error (connection,
+                                      MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                      TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                      "insert_close_request");
+      return qs;
+    }
+    if (qs <= 0)
+    {
+      GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR == qs);
+      return qs;
+    }
+  }
+  return qs;
+}
+
+
+MHD_RESULT
+TEH_handler_reserves_close (struct TEH_RequestContext *rc,
+                            const struct TALER_ReservePublicKeyP *reserve_pub,
+                            const json_t *root)
+{
+  struct ReserveCloseContext rcc = {
+    .payto_uri = NULL,
+    .reserve_pub = reserve_pub
+  };
+  MHD_RESULT mhd_ret;
+  struct GNUNET_JSON_Specification spec[] = {
+    GNUNET_JSON_spec_timestamp ("request_timestamp",
+                                &rcc.timestamp),
+    GNUNET_JSON_spec_allow_null (
+      GNUNET_JSON_spec_string ("payto_uri",
+                               &rcc.payto_uri),
+      NULL),
+    GNUNET_JSON_spec_fixed_auto ("reserve_sig",
+                                 &rcc.reserve_sig),
+    GNUNET_JSON_spec_end ()
+  };
+
+  {
+    enum GNUNET_GenericReturnValue res;
+
+    res = TALER_MHD_parse_json_data (rc->connection,
+                                     root,
+                                     spec);
+    if (GNUNET_SYSERR == res)
+    {
+      GNUNET_break (0);
+      return MHD_NO; /* hard failure */
+    }
+    if (GNUNET_NO == res)
+    {
+      GNUNET_break_op (0);
+      return MHD_YES; /* failure */
+    }
+  }
+
+  {
+    struct GNUNET_TIME_Timestamp now;
+
+    now = GNUNET_TIME_timestamp_get ();
+    if (! GNUNET_TIME_absolute_approx_eq (now.abs_time,
+                                          rcc.timestamp.abs_time,
+                                          TIMESTAMP_TOLERANCE))
+    {
+      GNUNET_break_op (0);
+      return TALER_MHD_reply_with_error (rc->connection,
+                                         MHD_HTTP_BAD_REQUEST,
+                                         TALER_EC_EXCHANGE_GENERIC_CLOCK_SKEW,
+                                         NULL);
+    }
+  }
+
+  {
+    struct TEH_KeyStateHandle *keys;
+
+    keys = TEH_keys_get_state ();
+    if (NULL == keys)
+    {
+      GNUNET_break (0);
+      GNUNET_JSON_parse_free (spec);
+      return TALER_MHD_reply_with_error (rc->connection,
+                                         MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                         
TALER_EC_EXCHANGE_GENERIC_KEYS_MISSING,
+                                         NULL);
+    }
+    rcc.gf = TEH_keys_global_fee_by_time (keys,
+                                          rcc.timestamp);
+  }
+  if (NULL == rcc.gf)
+  {
+    GNUNET_break (0);
+    return TALER_MHD_reply_with_error (rc->connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       
TALER_EC_EXCHANGE_GENERIC_BAD_CONFIGURATION,
+                                       NULL);
+  }
+  if (NULL != rcc.payto_uri)
+    TALER_payto_hash (&rcc.payto_uri,
+                      &rcc.h_payto);
+  if (GNUNET_OK !=
+      TALER_wallet_reserve_close_verify (rcc.timestamp,
+                                         &rcc.h_payto,
+                                         reserve_pub,
+                                         &rcc.reserve_sig))
+  {
+    GNUNET_break_op (0);
+    return TALER_MHD_reply_with_error (rc->connection,
+                                       MHD_HTTP_FORBIDDEN,
+                                       
TALER_EC_EXCHANGE_RESERVES_CLOSE_BAD_SIGNATURE,
+                                       NULL);
+  }
+  if (GNUNET_OK !=
+      TEH_DB_run_transaction (rc->connection,
+                              "reserve close",
+                              TEH_MT_REQUEST_OTHER,
+                              &mhd_ret,
+                              &reserve_close_transaction,
+                              &rcc))
+  {
+    return mhd_ret;
+  }
+  return reply_reserve_close_success (rc->connection,
+                                      &rcc);
+}
+
+
+/* end of taler-exchange-httpd_reserves_close.c */
diff --git a/src/exchange/taler-exchange-httpd_reserves_get_attest.c 
b/src/exchange/taler-exchange-httpd_reserves_get_attest.c
new file mode 100644
index 00000000..7d98babc
--- /dev/null
+++ b/src/exchange/taler-exchange-httpd_reserves_get_attest.c
@@ -0,0 +1,146 @@
+/*
+  This file is part of TALER
+  Copyright (C) 2014-2022 Taler Systems SA
+
+  TALER is free software; you can redistribute it and/or modify it under the
+  terms of the GNU Affero General Public License as published by the Free 
Software
+  Foundation; either version 3, or (at your option) any later version.
+
+  TALER is distributed in the hope that it will be useful, but WITHOUT ANY
+  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+  A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more 
details.
+
+  You should have received a copy of the GNU Affero General Public License 
along with
+  TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
+*/
+/**
+ * @file taler-exchange-httpd_reserves_get_attest.c
+ * @brief Handle GET /reserves/$RESERVE_PUB/attest requests
+ * @author Christian Grothoff
+ */
+#include "platform.h"
+#include <gnunet/gnunet_util_lib.h>
+#include <jansson.h>
+#include "taler_mhd_lib.h"
+#include "taler_json_lib.h"
+#include "taler_dbevents.h"
+#include "taler-exchange-httpd_keys.h"
+#include "taler-exchange-httpd_reserves_get_attest.h"
+#include "taler-exchange-httpd_responses.h"
+
+
+/**
+ * Closure for #reserve_attest_transaction.
+ */
+struct ReserveAttestContext
+{
+  /**
+   * Public key of the reserve the inquiry is about.
+   */
+  struct TALER_ReservePublicKeyP reserve_pub;
+
+  /**
+   * Available attributes.
+   */
+  json_t *attributes;
+
+  /**
+   * Set to true if we did not find the reserve.
+   */
+  bool not_found;
+};
+
+
+/**
+ * Function implementing GET /reserves/$RID/attest transaction.
+ * Execute a /reserves/ get attest.  Given the public key of a reserve,
+ * return the associated transaction attest.  Runs the
+ * transaction logic; IF it returns a non-error code, the transaction
+ * logic MUST NOT queue a MHD response.  IF it returns an hard error,
+ * the transaction logic MUST queue a MHD response and set @a mhd_ret.
+ * IF it returns the soft error code, the function MAY be called again
+ * to retry and MUST not queue a MHD response.
+ *
+ * @param cls a `struct ReserveAttestContext *`
+ * @param connection MHD request which triggered the transaction
+ * @param[out] mhd_ret set to MHD response status for @a connection,
+ *             if transaction failed (!)
+ * @return transaction status
+ */
+static enum GNUNET_DB_QueryStatus
+reserve_attest_transaction (void *cls,
+                            struct MHD_Connection *connection,
+                            MHD_RESULT *mhd_ret)
+{
+  struct ReserveAttestContext *rsc = cls;
+  enum GNUNET_DB_QueryStatus qs;
+
+  qs = TEH_plugin->get_reserve_attributes (TEH_plugin->cls,
+                                           &rsc->reserve_pub,
+                                           &rsc->attributes);
+  if (GNUNET_DB_STATUS_HARD_ERROR == qs)
+  {
+    GNUNET_break (0);
+    *mhd_ret
+      = TALER_MHD_reply_with_error (connection,
+                                    MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                    TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                    "get_reserve_attributes");
+  }
+  if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
+    rsc->not_found = true;
+  if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs)
+    rsc->not_found = false;
+  return qs;
+}
+
+
+MHD_RESULT
+TEH_handler_reserves_get_attest (struct TEH_RequestContext *rc,
+                                 const char *const args[1])
+{
+  struct ReserveAttestContext rsc;
+
+  if (GNUNET_OK !=
+      GNUNET_STRINGS_string_to_data (args[0],
+                                     strlen (args[0]),
+                                     &rsc.reserve_pub,
+                                     sizeof (rsc.reserve_pub)))
+  {
+    GNUNET_break_op (0);
+    return TALER_MHD_reply_with_error (rc->connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       
TALER_EC_MERCHANT_GENERIC_RESERVE_PUB_MALFORMED,
+                                       args[0]);
+  }
+  {
+    MHD_RESULT mhd_ret;
+
+    if (GNUNET_OK !=
+        TEH_DB_run_transaction (rc->connection,
+                                "get-attestable",
+                                TEH_MT_REQUEST_OTHER,
+                                &mhd_ret,
+                                &reserve_attest_transaction,
+                                &rsc))
+    {
+      return mhd_ret;
+    }
+  }
+  /* generate proper response */
+  if (rsc.not_found)
+  {
+    return TALER_MHD_reply_with_error (rc->connection,
+                                       MHD_HTTP_NOT_FOUND,
+                                       
TALER_EC_EXCHANGE_RESERVES_STATUS_UNKNOWN,
+                                       args[0]);
+  }
+  return TALER_MHD_REPLY_JSON_PACK (
+    rc->connection,
+    MHD_HTTP_OK,
+    TALER_JSON_pack_object_steal ("attributes",
+                                  &rsc.attributes));
+}
+
+
+/* end of taler-exchange-httpd_reserves_get_attest.c */
diff --git a/src/exchange/taler-exchange-httpd_reserves_history.h 
b/src/exchange/taler-exchange-httpd_reserves_history.h
index 9a2a9378..e02cb4d9 100644
--- a/src/exchange/taler-exchange-httpd_reserves_history.h
+++ b/src/exchange/taler-exchange-httpd_reserves_history.h
@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2020 Taler Systems SA
+  Copyright (C) 2014-2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free 
Software
@@ -15,9 +15,8 @@
 */
 /**
  * @file taler-exchange-httpd_reserves_history.h
- * @brief Handle /reserves/$RESERVE_PUB HISTORY requests
+ * @brief Handle /reserves/$RESERVE_PUB/history requests
  * @author Florian Dold
- * @author Benedikt Mueller
  * @author Christian Grothoff
  */
 #ifndef TALER_EXCHANGE_HTTPD_RESERVES_HISTORY_H
diff --git a/src/exchange/taler-exchange-httpd_reserves_open.c 
b/src/exchange/taler-exchange-httpd_reserves_open.c
new file mode 100644
index 00000000..9ef736f9
--- /dev/null
+++ b/src/exchange/taler-exchange-httpd_reserves_open.c
@@ -0,0 +1,281 @@
+/*
+  This file is part of TALER
+  Copyright (C) 2014-2022 Taler Systems SA
+
+  TALER is free software; you can redistribute it and/or modify it under the
+  terms of the GNU Affero General Public License as published by the Free 
Software
+  Foundation; either version 3, or (at your option) any later version.
+
+  TALER is distributed in the hope that it will be useful, but WITHOUT ANY
+  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+  A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more 
details.
+
+  You should have received a copy of the GNU Affero General Public License 
along with
+  TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
+*/
+/**
+ * @file taler-exchange-httpd_reserves_open.c
+ * @brief Handle /reserves/$RESERVE_PUB/open requests
+ * @author Christian Grothoff
+ */
+#include "platform.h"
+#include <gnunet/gnunet_util_lib.h>
+#include <jansson.h>
+#include "taler_mhd_lib.h"
+#include "taler_json_lib.h"
+#include "taler_dbevents.h"
+#include "taler-exchange-httpd_keys.h"
+#include "taler-exchange-httpd_reserves_open.h"
+#include "taler-exchange-httpd_responses.h"
+
+
+/**
+ * How far do we allow a client's time to be off when
+ * checking the request timestamp?
+ */
+#define TIMESTAMP_TOLERANCE \
+  GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_MINUTES, 15)
+
+
+/**
+ * Closure for #reserve_open_transaction.
+ */
+struct ReserveOpenContext
+{
+  /**
+   * Public key of the reserve the inquiry is about.
+   */
+  const struct TALER_ReservePublicKeyP *reserve_pub;
+
+  /**
+   * Timestamp of the request.
+   */
+  struct GNUNET_TIME_Timestamp timestamp;
+
+  /**
+   * Client signature approving the request.
+   */
+  struct TALER_ReserveSignatureP reserve_sig;
+
+  /**
+   * Open of the reserve, set in the callback.
+   */
+  struct TALER_EXCHANGEDB_ReserveOpen *rh;
+
+  /**
+   * Global fees applying to the request.
+   */
+  const struct TEH_GlobalFee *gf;
+
+  /**
+   * Current reserve balance.
+   */
+  struct TALER_Amount balance;
+};
+
+
+/**
+ * Send reserve open to client.
+ *
+ * @param connection connection to the client
+ * @param rhc reserve open to return
+ * @return MHD result code
+ */
+static MHD_RESULT
+reply_reserve_open_success (struct MHD_Connection *connection,
+                            const struct ReserveOpenContext *rhc)
+{
+  const struct TALER_EXCHANGEDB_ReserveOpen *rh = rhc->rh;
+
+  return TALER_MHD_REPLY_JSON_PACK (
+    connection,
+    MHD_HTTP_OK,
+    TALER_JSON_pack_amount ("balance",
+                            &rhc->balance));
+}
+
+
+/**
+ * Function implementing /reserves/$RID/open transaction.  Given the public
+ * key of a reserve, return the associated transaction open.  Runs the
+ * transaction logic; IF it returns a non-error code, the transaction logic
+ * MUST NOT queue a MHD response.  IF it returns an hard error, the
+ * transaction logic MUST queue a MHD response and set @a mhd_ret.  IF it
+ * returns the soft error code, the function MAY be called again to retry and
+ * MUST not queue a MHD response.
+ *
+ * @param cls a `struct ReserveOpenContext *`
+ * @param connection MHD request which triggered the transaction
+ * @param[out] mhd_ret set to MHD response status for @a connection,
+ *             if transaction failed (!); unused
+ * @return transaction status
+ */
+static enum GNUNET_DB_QueryStatus
+reserve_open_transaction (void *cls,
+                          struct MHD_Connection *connection,
+                          MHD_RESULT *mhd_ret)
+{
+  struct ReserveOpenContext *rsc = cls;
+  enum GNUNET_DB_QueryStatus qs;
+
+  if (! TALER_amount_is_zero (&rsc->gf->fees.open))
+  {
+    bool balance_ok = false;
+    bool idempotent = true;
+
+    qs = TEH_plugin->insert_open_request (TEH_plugin->cls,
+                                          rsc->reserve_pub,
+                                          &rsc->reserve_sig,
+                                          rsc->timestamp,
+                                          &rsc->gf->fees.open,
+                                          &balance_ok,
+                                          &idempotent);
+    if (GNUNET_DB_STATUS_HARD_ERROR == qs)
+    {
+      GNUNET_break (0);
+      *mhd_ret
+        = TALER_MHD_reply_with_error (connection,
+                                      MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                      TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                      "get_reserve_open");
+    }
+    if (qs <= 0)
+    {
+      GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR == qs);
+      return qs;
+    }
+    if (! balance_ok)
+    {
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_CONFLICT,
+                                         
TALER_EC_EXCHANGE_WITHDRAW_OPEN_ERROR_INSUFFICIENT_FUNDS,
+                                         NULL);
+    }
+    if (idempotent)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                  "Idempotent /reserves/open request observed. Is caching 
working?\n");
+    }
+  }
+  qs = TEH_plugin->get_reserve_open (TEH_plugin->cls,
+                                     rsc->reserve_pub,
+                                     &rsc->balance,
+                                     &rsc->rh);
+  if (GNUNET_DB_STATUS_HARD_ERROR == qs)
+  {
+    GNUNET_break (0);
+    *mhd_ret
+      = TALER_MHD_reply_with_error (connection,
+                                    MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                    TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                    "get_reserve_open");
+  }
+  return qs;
+}
+
+
+MHD_RESULT
+TEH_handler_reserves_open (struct TEH_RequestContext *rc,
+                           const struct TALER_ReservePublicKeyP *reserve_pub,
+                           const json_t *root)
+{
+  struct ReserveOpenContext rsc;
+  MHD_RESULT mhd_ret;
+  struct GNUNET_JSON_Specification spec[] = {
+    GNUNET_JSON_spec_timestamp ("request_timestamp",
+                                &rsc.timestamp),
+    GNUNET_JSON_spec_fixed_auto ("reserve_sig",
+                                 &rsc.reserve_sig),
+    GNUNET_JSON_spec_end ()
+  };
+  struct GNUNET_TIME_Timestamp now;
+
+  rsc.reserve_pub = reserve_pub;
+  {
+    enum GNUNET_GenericReturnValue res;
+
+    res = TALER_MHD_parse_json_data (rc->connection,
+                                     root,
+                                     spec);
+    if (GNUNET_SYSERR == res)
+    {
+      GNUNET_break (0);
+      return MHD_NO; /* hard failure */
+    }
+    if (GNUNET_NO == res)
+    {
+      GNUNET_break_op (0);
+      return MHD_YES; /* failure */
+    }
+  }
+  now = GNUNET_TIME_timestamp_get ();
+  if (! GNUNET_TIME_absolute_approx_eq (now.abs_time,
+                                        rsc.timestamp.abs_time,
+                                        TIMESTAMP_TOLERANCE))
+  {
+    GNUNET_break_op (0);
+    return TALER_MHD_reply_with_error (rc->connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_EXCHANGE_GENERIC_CLOCK_SKEW,
+                                       NULL);
+  }
+  {
+    struct TEH_KeyStateHandle *keys;
+
+    keys = TEH_keys_get_state ();
+    if (NULL == keys)
+    {
+      GNUNET_break (0);
+      GNUNET_JSON_parse_free (spec);
+      return TALER_MHD_reply_with_error (rc->connection,
+                                         MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                         
TALER_EC_EXCHANGE_GENERIC_KEYS_MISSING,
+                                         NULL);
+    }
+    rsc.gf = TEH_keys_global_fee_by_time (keys,
+                                          rsc.timestamp);
+  }
+  if (NULL == rsc.gf)
+  {
+    GNUNET_break (0);
+    return TALER_MHD_reply_with_error (rc->connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       
TALER_EC_EXCHANGE_GENERIC_BAD_CONFIGURATION,
+                                       NULL);
+  }
+  if (GNUNET_OK !=
+      TALER_wallet_reserve_open_verify (rsc.timestamp,
+                                        &rsc.gf->fees.open,
+                                        reserve_pub,
+                                        &rsc.reserve_sig))
+  {
+    GNUNET_break_op (0);
+    return TALER_MHD_reply_with_error (rc->connection,
+                                       MHD_HTTP_FORBIDDEN,
+                                       
TALER_EC_EXCHANGE_RESERVES_OPEN_BAD_SIGNATURE,
+                                       NULL);
+  }
+  rsc.rh = NULL;
+  if (GNUNET_OK !=
+      TEH_DB_run_transaction (rc->connection,
+                              "reserve open",
+                              TEH_MT_REQUEST_OTHER,
+                              &mhd_ret,
+                              &reserve_open_transaction,
+                              &rsc))
+  {
+    return mhd_ret;
+  }
+  if (NULL == rsc.rh)
+  {
+    return TALER_MHD_reply_with_error (rc->connection,
+                                       MHD_HTTP_NOT_FOUND,
+                                       
TALER_EC_EXCHANGE_RESERVES_STATUS_UNKNOWN,
+                                       NULL);
+  }
+  return reply_reserve_open_success (rc->connection,
+                                     &rsc);
+}
+
+
+/* end of taler-exchange-httpd_reserves_open.c */
diff --git a/src/exchange/taler-exchange-httpd_reserves_history.h 
b/src/exchange/taler-exchange-httpd_reserves_attest.h
similarity index 63%
copy from src/exchange/taler-exchange-httpd_reserves_history.h
copy to src/exchange/taler-exchange-httpd_reserves_attest.h
index 9a2a9378..d5ec8fd7 100644
--- a/src/exchange/taler-exchange-httpd_reserves_history.h
+++ b/src/exchange/taler-exchange-httpd_reserves_attest.h
@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2020 Taler Systems SA
+  Copyright (C) 2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free 
Software
@@ -14,21 +14,19 @@
   TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */
 /**
- * @file taler-exchange-httpd_reserves_history.h
- * @brief Handle /reserves/$RESERVE_PUB HISTORY requests
- * @author Florian Dold
- * @author Benedikt Mueller
+ * @file taler-exchange-httpd_reserves_attest.h
+ * @brief Handle /reserves/$RESERVE_PUB/attest requests
  * @author Christian Grothoff
  */
-#ifndef TALER_EXCHANGE_HTTPD_RESERVES_HISTORY_H
-#define TALER_EXCHANGE_HTTPD_RESERVES_HISTORY_H
+#ifndef TALER_EXCHANGE_HTTPD_RESERVES_ATTEST_H
+#define TALER_EXCHANGE_HTTPD_RESERVES_ATTEST_H
 
 #include <microhttpd.h>
 #include "taler-exchange-httpd.h"
 
 
 /**
  * Handle a POST "/reserves/$RID/history" request.
  *
  * @param rc request context
  * @param reserve_pub public key of the reserve
@@ -36,8 +34,8 @@
  * @return MHD result code
  */
 MHD_RESULT
-TEH_handler_reserves_history (struct TEH_RequestContext *rc,
-                              const struct TALER_ReservePublicKeyP 
*reserve_pub,
-                              const json_t *root);
+TEH_handler_reserves_attest (struct TEH_RequestContext *rc,
+                             const struct TALER_ReservePublicKeyP *reserve_pub,
+                             const json_t *root);
 
 #endif
@@ -14,21 +14,19 @@
 
 /**
- * Handle a POST "/reserves/$RID/history" request.
+ * Handle a POST "/reserves/$RID/attest" request.
  *
  * @param rc request context
diff --git a/src/exchange/taler-exchange-httpd_reserves_history.h 
b/src/exchange/taler-exchange-httpd_reserves_close.h
similarity index 63%
copy from src/exchange/taler-exchange-httpd_reserves_history.h
copy to src/exchange/taler-exchange-httpd_reserves_close.h
index 9a2a9378..4c70b17c 100644
--- a/src/exchange/taler-exchange-httpd_reserves_history.h
+++ b/src/exchange/taler-exchange-httpd_reserves_close.h
@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2020 Taler Systems SA
+  Copyright (C) 2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free 
Software
@@ -14,21 +14,19 @@
   TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */
 /**
- * @file taler-exchange-httpd_reserves_history.h
- * @brief Handle /reserves/$RESERVE_PUB HISTORY requests
- * @author Florian Dold
- * @author Benedikt Mueller
+ * @file taler-exchange-httpd_reserves_close.h
+ * @brief Handle /reserves/$RESERVE_PUB/close requests
  * @author Christian Grothoff
  */
-#ifndef TALER_EXCHANGE_HTTPD_RESERVES_HISTORY_H
-#define TALER_EXCHANGE_HTTPD_RESERVES_HISTORY_H
+#ifndef TALER_EXCHANGE_HTTPD_RESERVES_CLOSE_H
+#define TALER_EXCHANGE_HTTPD_RESERVES_CLOSE_H
 
 #include <microhttpd.h>
 #include "taler-exchange-httpd.h"
 
 
 /**
  * Handle a POST "/reserves/$RID/history" request.
  *
  * @param rc request context
  * @param reserve_pub public key of the reserve
@@ -36,8 +34,8 @@
  * @return MHD result code
  */
 MHD_RESULT
-TEH_handler_reserves_history (struct TEH_RequestContext *rc,
-                              const struct TALER_ReservePublicKeyP 
*reserve_pub,
-                              const json_t *root);
+TEH_handler_reserves_close (struct TEH_RequestContext *rc,
+                            const struct TALER_ReservePublicKeyP *reserve_pub,
+                            const json_t *root);
 
 #endif
@@ -14,21 +14,19 @@
 
 /**
- * Handle a POST "/reserves/$RID/history" request.
+ * Handle a POST "/reserves/$RID/close" request.
  *
  * @param rc request context
diff --git a/src/exchange/taler-exchange-httpd_reserves_history.h 
b/src/exchange/taler-exchange-httpd_reserves_open.h
similarity index 63%
copy from src/exchange/taler-exchange-httpd_reserves_history.h
copy to src/exchange/taler-exchange-httpd_reserves_open.h
index 9a2a9378..e28c22c0 100644
--- a/src/exchange/taler-exchange-httpd_reserves_history.h
+++ b/src/exchange/taler-exchange-httpd_reserves_open.h
@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2020 Taler Systems SA
+  Copyright (C) 2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free 
Software
@@ -14,21 +14,19 @@
   TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */
 /**
- * @file taler-exchange-httpd_reserves_history.h
- * @brief Handle /reserves/$RESERVE_PUB HISTORY requests
- * @author Florian Dold
- * @author Benedikt Mueller
+ * @file taler-exchange-httpd_reserves_open.h
+ * @brief Handle /reserves/$RESERVE_PUB/open requests
  * @author Christian Grothoff
  */
-#ifndef TALER_EXCHANGE_HTTPD_RESERVES_HISTORY_H
-#define TALER_EXCHANGE_HTTPD_RESERVES_HISTORY_H
+#ifndef TALER_EXCHANGE_HTTPD_RESERVES_OPEN_H
+#define TALER_EXCHANGE_HTTPD_RESERVES_OPEN_H
 
 #include <microhttpd.h>
 #include "taler-exchange-httpd.h"
 
 
 /**
  * Handle a POST "/reserves/$RID/history" request.
  *
  * @param rc request context
  * @param reserve_pub public key of the reserve
@@ -36,8 +34,8 @@
  * @return MHD result code
  */
 MHD_RESULT
-TEH_handler_reserves_history (struct TEH_RequestContext *rc,
-                              const struct TALER_ReservePublicKeyP 
*reserve_pub,
-                              const json_t *root);
+TEH_handler_reserves_open (struct TEH_RequestContext *rc,
+                           const struct TALER_ReservePublicKeyP *reserve_pub,
+                           const json_t *root);
 
 #endif
@@ -14,21 +14,19 @@
 
 /**
- * Handle a POST "/reserves/$RID/history" request.
+ * Handle a POST "/reserves/$RID/open" request.
  *
  * @param rc request context

-- 
To stop receiving notification emails like this one, please contact
gnunet@gnunet.org.



reply via email to

[Prev in Thread] Current Thread [Next in Thread]